#!/usr/bin/env perl
use strict;
use AnyEvent;
use AnyEvent::Handle;
use AnyEvent::HTTP;

main() unless caller();

sub main {
    my $cv = AnyEvent->condvar;

    # 標準入力からURLを1行ずつもらうので、STDINを監視する
    # AnyEvent::Handleオブジェクトを作成する
    my $handle = AnyEvent::Handle->new(
        fh       => \*STDIN,
        # エラー、もしくはEOF時に$cvに終了通知を送る
        on_eof   => sub { $cv->end },
        on_error => sub { $cv->end },
    );

    # 最後の待ちを有効にするために1回beginを呼んでおく
    $cv->begin();

    my $w;
    my $read_stdin; $read_stdin = sub {
        $handle->push_read( line => sub {
            my ($handle, $url) = @_;
            chomp $url;
            if ($url) {
                $cv->begin(); # このURLを処理している間は
                              # $cvの条件を満たさないように
                              # フラグを立てておく

                # AnyEvent::HTTPを使ってリクエストを送る
                http_get $url, sub {
                    my ($body, $headers) = @_;
                    print " + $headers->{URL} -> " .
                        "$headers->{Status}\n";

                    # URLを取得したのでフラグを落とす
                    $cv->end();
                };
            }
            # 1行読んだので、もう1行読むために$read_stdinを
            # イベントキューにいれてもらう
            $w = AnyEvent->timer(after => 0, cb => $read_stdin);
        } );
    };

    $w = AnyEvent->timer(after => 0, cb => $read_stdin);

    # $cv->end()がbegin()の回数分呼ばれるまで待つ
    $cv->recv();
}

